/* 
    CMx2 ScAnCaDe - Scenario Analyser and Campaign Decompiler
    
    Analysing scenarios and campaigns and creating sortable listings 
    containing meta-information about the analysed scenarios/campaigns.
    Also, extract all scenarios belonging to a campaign from the
    campaign file, enabling stand-alone play of these battles.
    Programmed to support the players and scenario designers of the 
    Combat Mission - CMx2 engine WW2 computer games produced by
    Battlefront Inc. (www.battlefront.com)
    
    Copyright (C) 2014  Michael Wellhfer (michael@wellhoefer.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    Change History:
    
    ---------------------------------------------------------------------
    | VERSION |   DATE     |             DESCRIPTION                    |
    |---------|------------|--------------------------------------------|
    |   1.0   | 10/06/2013 | Initial release version.                   |
    |---------|------------|--------------------------------------------|
    |   1.4   | 22/02/2014 | Added required Modules information.        |
    |---------|------------|--------------------------------------------|
    |  1.5#1  | 09/03/2014 | Added number of campaign ends information. |
    |---------|------------|--------------------------------------------|
    |  1.5#2  | 07/04/2014 | Added campaign tree filename.              |
    |---------|------------|--------------------------------------------|
    |   1.8   | 23/12/2014 | Added optional Modules information.        |
    |---------|------------|--------------------------------------------|
*/

package Data;

public class CMx2_Campaign {

	private int numberOfScenarios = 0;
	
	private int numberOfCampaignEnds = 0;
	
	private String title;
	
	private String createdwithVersion;
	
	private String description;
	
	private String fileName;
	
	private String campaignTreeFilename;
	
    private int numberOfModulesRequired;
	
	private String firstModuleRequired;
	private String secondModuleRequired;
	private String thirdModuleRequired;
	private String fourthModuleRequired;
	
	private String optionalModulesRequired;
	
	private boolean optionalModulesDefined = false;
	
	private CMx2_CampaignDecisionBlock[] decisionBlockArray;
	
	private CMx2_Scenario[] scenarioArray;
	
	public CMx2_Campaign() {
		
		title 		 	   		= "Not defined!";
		createdwithVersion 		= "Unknown!";
		description  	   		= "Not defined!";
		numberOfModulesRequired = 0;
		firstModuleRequired     = "Not defined!";
		secondModuleRequired    = "Not defined!"; 
		thirdModuleRequired     = "Not defined!";
		fourthModuleRequired    = "Not defined!";
		optionalModulesRequired = "Not defined!";
		
	}
	
	public void setNumberOfScenarios(int number) {
		
		this.numberOfScenarios = number;
		
		decisionBlockArray = new CMx2_CampaignDecisionBlock[number];
		
		scenarioArray = new CMx2_Scenario[number];
		
	}
	
	public int getNumberOfScenarios() {
		
		return this.numberOfScenarios;
		
	}
	
	public void setNumberOfCampaignEnds(int number) {
		
		this.numberOfCampaignEnds = number;
		
	}
	
	public int getNumberOfCampaignEnds() {
		
		return this.numberOfCampaignEnds;
		
	}
	
	public void setTitle(String scenTitle) {
		
		this.title = scenTitle;
		
	}
	
	public String getTitle() {
		
		return this.title;
		
	}
	
	public void setCreatedwithVersion(String version) {
		
		this.createdwithVersion = version;
		
	}
	
	public String getCreatedwithVersion() {
		
		return this.createdwithVersion;
		
	}
	
	public void setDescription(String scenDescription) {
		
		this.description = scenDescription;
		
	}
	
	public String getDescription() {
		
		return this.description;
		
	}
	
	public void addCampaignDecisionBlock(CMx2_CampaignDecisionBlock block, int numberOfBattle){
		
		decisionBlockArray[numberOfBattle] = block;
		
	}
	
	public void addCampaignScenario(CMx2_Scenario scen, int numberOfBattle){
		
		scenarioArray[numberOfBattle] = scen;
		
	}
	
	public CMx2_CampaignDecisionBlock getCMBNCampaignDecisionBlock(int numberOfBattle){
		
		return decisionBlockArray[numberOfBattle];
		
	}
	
	public CMx2_Scenario getCampaignScenario(int numberOfBattle){
		
		return scenarioArray[numberOfBattle];
		
	}
	
	public void setfileName(String camFileName) {
		
		this.fileName = camFileName;
		
	}
	
	public String getfileName() {
		
		return this.fileName;
		
	}
	
	public void setCampaignTreeFilename(String treeFilename){
		
		this.campaignTreeFilename = treeFilename;
		
	}
	
	public String getCampaignTreeFilename(){
		
		return this.campaignTreeFilename;
		
	}
	
    public void setNumberOfModulesRequired(int numberOfModules) {
		
		this.numberOfModulesRequired = numberOfModules;
		
	}
	
	public int getNumberOfModulesRequired() {
		
		return this.numberOfModulesRequired;
		
	}
	
	public void setFirstModuleRequired(String requiredModule) {
		
		this.firstModuleRequired = requiredModule;
		
	}
	
	public String getFirstModuleRequired() {
		
		return this.firstModuleRequired;
		
	}
	
	public void setSecondModuleRequired(String requiredModule) {
		
		this.secondModuleRequired = requiredModule;
		
	}
	
	public String getSecondModuleRequired() {
		
		return this.secondModuleRequired;
		
	}
	
	public void setThirdModuleRequired(String requiredModule) {
		
		this.thirdModuleRequired = requiredModule;
		
	}
	
	public String getThirdModuleRequired() {
		
		return this.thirdModuleRequired;
		
	}
	
	public void setFourthModuleRequired(String requiredModule) {
		
		this.fourthModuleRequired = requiredModule;
		
	}
	
	public String getFourthModuleRequired() {
		
		return this.fourthModuleRequired;
		
	}
	
	public void setOptionalModulesRequired(String optionalModules) {
		
		this.optionalModulesRequired = optionalModules;
		
		optionalModulesDefined = true;
		
	}
	
	public String getOptionalModulesRequired() {
		
		return this.optionalModulesRequired;
		
	}
	
	public boolean getOptionalModulesDefined() {
		
		return this.optionalModulesDefined;
		
	}
	
}
