/* 
    CMx2 ScAnCaDe - Scenario Analyser and Campaign Decompiler
    
    Analysing scenarios and campaigns and creating sortable listings 
    containing meta-information about the analysed scenarios/campaigns.
    Also, extract all scenarios belonging to a campaign from the
    campaign file, enabling stand-alone play of these battles.
    Programmed to support the players and scenario designers of the 
    Combat Mission - CMx2 engine WW2 computer games produced by
    Battlefront Inc. (www.battlefront.com)
    
    Copyright (C) 2014  Michael Wellhfer (michael@wellhoefer.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    Change History:
    
    ---------------------------------------------------------------------
    | VERSION |   DATE     |             DESCRIPTION                    |
    |---------|------------|--------------------------------------------|
    |   1.0   | 10/06/2013 | Initial release version.                   |
    |---------|------------|--------------------------------------------|
    |   1.5   | 23/02/2014 | Added indication if internalBattleName has |
    |         |            | been defined.                              |
    |---------|------------|--------------------------------------------|
*/

package Data;

public class CMx2_CampaignDecisionBlock {

	private String internalBattleName;
	
	private String winThreshold;
	
	private String shortWinThreshold;
	
	private int nextBattleWin;
	
	private int nextBattleLose;
	
	private String minimumRequiredCampaignVictory;
	
	private String shortMinimumRequiredCampaignVictory;
	
	private String maximumAllowedCampaignVictory;
	
	private String shortMaximumAllowedCampaignVictory;
	
	private int blueRefit;
	
	private int blueRepairVehicle;
	
	private int blueResupply;
	
	private int blueRest;
	
	private int redRefit;
	
	private int redRepairVehicle;
	
	private int redResupply;
	
	private int redRest;
	
	private boolean containsScenario;
	
	private boolean internalBattleNameDefined;
	
	public CMx2_CampaignDecisionBlock(){
		
		internalBattleName 				= "Not defined yet!";
		
		winThreshold 					= "Not defined yet!";
		minimumRequiredCampaignVictory  = "Not defined yet!";
		maximumAllowedCampaignVictory	= "Not defined yet!";
		
		shortWinThreshold 				= "Not defined yet!";
		shortMinimumRequiredCampaignVictory  = "Not defined yet!";
		shortMaximumAllowedCampaignVictory	= "Not defined yet!";
		
		nextBattleWin 		= 0;
		nextBattleLose 		= 0;
		
		blueRefit			= 0;
		blueRepairVehicle 	= 0;
		blueResupply		= 0;
		blueRest			= 0;
		
		redRefit			= 0;
		redRepairVehicle 	= 0;
		redResupply			= 0;
		redRest				= 0;
		
		containsScenario = false;
		internalBattleNameDefined = false;
		
	}
	
	public String getinternalBattleName() {
		
		return this.internalBattleName;
		
	}
	
	public void setinternalBattleName(String battleName) {
		
		this.internalBattleName = battleName;
		
	}
	
	public String getwinThreshold() {
		
		return this.winThreshold;
		
	}
	
	public String getShortWinThreshold(){
		
		return this.shortWinThreshold;
		
	}
	
	public void setwinThreshold(String[] winthreshold) {
		
		this.winThreshold = winthreshold[0];
		this.shortWinThreshold = winthreshold[1];
		
	}
	
	public String getminimumRequiredCampaignVictory() {
		
		return this.minimumRequiredCampaignVictory;
		
	}
	
	public String getShortMinimumRequiredCampaignVictory(){
		
		return this.shortMinimumRequiredCampaignVictory;
		
	}
	
	public void setminimumRequiredCampaignVictory(String[] minimum_required_campaign_victory) {
		
		this.minimumRequiredCampaignVictory = minimum_required_campaign_victory[0];
		this.shortMinimumRequiredCampaignVictory = minimum_required_campaign_victory[1];
		
	}
	
	public String getmaximumAllowedCampaignVictory() {
		
		return this.maximumAllowedCampaignVictory;
		
	}
	
	public String getShortMaximumAllowedCampaignVictory(){
		
		return this.shortMaximumAllowedCampaignVictory;
		
	}
	
	public void setmaximumAllowedCampaignVictory(String[] maximum_allowed_campaign_victory) {
		
		this.maximumAllowedCampaignVictory = maximum_allowed_campaign_victory[0];
		this.shortMaximumAllowedCampaignVictory = maximum_allowed_campaign_victory[1];
		
	}
	
	public int getnextBattleWin() {
		
		return this.nextBattleWin;
		
	}
	
	public void setnextBattleWin(int next_battle_win){
		
		this.nextBattleWin = next_battle_win;
		
	}
	
	public int getnextBattleLose() {
		
		return this.nextBattleLose;
		
	}
	
	public void setnextBattleLose(int next_battle_lose){
		
		this.nextBattleLose = next_battle_lose;
		
	}
	
	public int getblueRefit() {
		
		return this.blueRefit;
		
	}
	
	public void setblueRefit(int blue_refit){
		
		this.blueRefit = blue_refit;
		
	}
	
	public int getblueRepairVehicle() {
		
		return this.blueRepairVehicle;
		
	}
	
	public void setblueRepairVehicle(int blue_repair_vehicle){
		
		this.blueRepairVehicle = blue_repair_vehicle;
		
	}
	
	public int getblueResupply() {
		
		return this.blueResupply;
		
	}
	
	public void setblueResupply(int blue_resupply){
		
		this.blueResupply = blue_resupply;
		
	}
	
	public int getblueRest() {
		
		return this.blueRest;
		
	}
	
	public void setblueRest(int blue_rest){
		
		this.blueRest = blue_rest;
		
	}
	
	public int getredRefit() {
		
		return this.redRefit;
		
	}
	
	public void setredRefit(int red_refit){
		
		this.redRefit = red_refit;
		
	}
	
	public int getredRepairVehicle() {
		
		return this.redRepairVehicle;
		
	}
	
	public void setredRepairVehicle(int red_repair_vehicle){
		
		this.redRepairVehicle = red_repair_vehicle;
		
	}
	
	public int getredResupply() {
		
		return this.redResupply;
		
	}
	
	public void setredResupply(int red_resupply){
		
		this.redResupply = red_resupply;
		
	}
	
	public int getredRest() {
		
		return this.redRest;
		
	}
	
	public void setredRest(int red_rest){
		
		this.redRest = red_rest;
		
	}
	
	public boolean getcontainsScenario() {
		
		return this.containsScenario;
		
	}
	
	public void setcontainsScenario(boolean contains_scenario){
		
		this.containsScenario = contains_scenario;
		
	}
	
	public boolean getInternalBattleNameDefined() {
		
		return this.internalBattleNameDefined;
		
	}
	
	public void setInternalBattleNameDefined(boolean name_defined) {
		
		this.internalBattleNameDefined = name_defined;
		
	}
	
}
