/* 
    CMx2 ScAnCaDe - Scenario Analyser and Campaign Decompiler
    
    Analysing scenarios and campaigns and creating sortable listings 
    containing meta-information about the analysed scenarios/campaigns.
    Also, extract all scenarios belonging to a campaign from the
    campaign file, enabling stand-alone play of these battles.
    Programmed to support the players and scenario designers of the 
    Combat Mission - CMx2 engine WW2 computer games produced by
    Battlefront Inc. (www.battlefront.com)
    
    Copyright (C) 2014  Michael Wellhfer (michael@wellhoefer.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    Change History:
    
    ---------------------------------------------------------------------
    | VERSION |   DATE     |             DESCRIPTION                    |
    |---------|------------|--------------------------------------------|
    |   1.0   | 10/06/2013 | Initial release version.                   |
    |---------|------------|--------------------------------------------|
    |   1.1   | 13/10/2013 | Added Region information.                  |
    |---------|------------|--------------------------------------------|
    |   1.3   | 15/12/2013 | Added Year information.                    |
    |---------|------------|--------------------------------------------|
    |   1.4   | 09/02/2014 | Added required Modules information.        |
    |---------|------------|--------------------------------------------|
    |   1.5   | 05/04/2014 | Added scenario specific Mod tag            |
    |         |            | information (for v3.0 / CMRT).             |
    |---------|------------|--------------------------------------------|
    |   1.8   | 22/11/2014 | Added optional Modules information.        |
    |---------|------------|--------------------------------------------|
*/

package Data;

import java.text.DecimalFormat;

public class CMx2_Scenario {

	private static int numberOfScenarios = 0;
	
	private String title;
	
	private String fileName;
	
	private String bitmapFileName;
	
	private String campaignBattleNumber;
	
	private int campaignOrderNumber;
	
	private String createdwithVersion;
	
	private String description;
	
	private String environment;
	
	private String battleType;
	
	private String daylight;
	
	private String battleSize;
	
	private String year;
	
	private String region;
	
	private String month;	
	
	private String day;
	
	private String time;
	
	private String battleLength;
	
	private String weather;
	
	private String temperature;
	
	private String groundCondition;
	
	private String earlyIntel;
	
	private String mapWidth;
	
	private String mapDepth;
	
	private String mapArea;
	
	private String sizeModifier;
	
	private boolean mapWidthSet = false;
	private boolean mapDepthSet = false;
	
	private int numberOfModulesRequired;
	
	private String firstModuleRequired;
	private String secondModuleRequired;
	private String thirdModuleRequired;
	private String fourthModuleRequired;
	
	private String optionalModulesRequired;
	
	private boolean optionalModulesDefined = false;
	
	private String[] modTags;
	
	public CMx2_Scenario() {
		
		title 		 			= "Not defined!";
		createdwithVersion      = "Unknown!";
		campaignBattleNumber	= "";
		description  			= "Not defined!";
		environment  			= "Not defined!";
		battleType 	 			= "Not defined!";
		daylight 	 			= "Not defined!";
		battleSize   			= "Not defined!";
		region                  = "Not defined!";
		month		 			= "Not defined!";
		day			 			= "Not defined!";
		time		 			= "Not defined!";
		battleLength 			= "Not defined!";
		weather		 			= "Not defined!";
		temperature  			= "Not defined!";
		groundCondition			= "Not defined!";
		earlyIntel				= "Not defined!";
		mapWidth				= "Not defined!";
		mapDepth				= "Not defined!";
		mapArea					= "Not defined!";
		sizeModifier            = "Not defined!";
		numberOfModulesRequired = 0;
		firstModuleRequired     = "Not defined!";
		secondModuleRequired    = "Not defined!"; 
		thirdModuleRequired     = "Not defined!";
		fourthModuleRequired    = "Not defined!";
		optionalModulesRequired = "Not defined!";
		
		numberOfScenarios++;
		
	}
	
	public int getNumberOfScenarios() {
		
		return numberOfScenarios;
		
	}
	
	public void setTitle(String scenTitle) {
		
		this.title = scenTitle;
		
	}
	
	public String getTitle() {
		
		return this.title;
		
	}
	
	public void setCreatedwithVersion(String version) {
		
		this.createdwithVersion = version;
		
	}
	
	public String getCreatedwithVersion() {
		
		return this.createdwithVersion;
		
	}
	
	public void setfileName(String scenFileName) {
		
		this.fileName = scenFileName;
		
	}
	
	public String getfileName() {
		
		return this.fileName;
		
	}
	
	public void setbitmapFileName(String bitmapFileName) {
		
		this.bitmapFileName = bitmapFileName;
		
	}
	
	public String getbitmapFileName() {
		
		return this.bitmapFileName;
		
	}
	
	public void setCampaignBattleNumber(String battleNumber) {
		
		this.campaignBattleNumber = battleNumber;
		
	}
	
	public String getCampaignBattleNumber() {
		
		return this.campaignBattleNumber;
		
	}
	
	public void setCampaignOrderNumber(int orderNumber){
		
		this.campaignOrderNumber = orderNumber;
		
	}
	
	public int getCampaignOrderNumber(){
		
		return this.campaignOrderNumber;
		
	}
	
	public void setDescription(String scenDescription) {
		
		this.description = scenDescription;
		
	}
	
	public String getDescription() {
		
		return this.description;
		
	}
	
	public void setEnvironment(String scenEnvironment) {
		
		this.environment = scenEnvironment;
		
	}
	
	public String getEnvironment() {
		
		return this.environment;
		
	}
	
	public void setBattleType(String scenBattleType) {
		
		this.battleType = scenBattleType;
		
	}
	
	public String getBattleType() {
		
		return this.battleType;
		
	}
	
	public void setDaylight(String scenDaylight) {
		
		this.daylight = scenDaylight;
		
	}
	
	public String getDaylight() {
		
		return this.daylight;
		
	}
	
	public void setBattleSize(String scenBattleSize) {
		
		this.battleSize = scenBattleSize;
		
	}
	
	public String getBattleSize() {
		
		return this.battleSize;
		
	}
	
	public void setYear(String scenYear) {
		
		this.year = scenYear;
		
	}
	
	public String getYear() {
		
		return year;
		
	}
	
	public void setMonth(String scenMonth) {
		
		this.month = scenMonth;
		
	}
	
	public String getMonth() {
		
		return this.month;
		
	}
	
	public void setRegion(String scenRegion) {
		
		this.region = scenRegion;
		
	}
	
	public String getRegion() {
		
		return this.region;
		
	}
	
	public void setDay(String scenDay) {
		
		this.day = scenDay;
		
	}
	
	public String getDay() {
		
		return this.day;
		
	}
	
	public void setTime(String scenTime) {
		
		this.time = scenTime;
		
	}
	
	public String getTime() {
		
		return this.time;
		
	}
	
	public void setBattleLength(String scenBattleLength) {
		
		this.battleLength = scenBattleLength;
		
	}
	
	public String getBattleLength() {
		
		return this.battleLength;
		
	}
	
	public void setWeather(String scenWeather) {
		
		this.weather = scenWeather;
		
	}
	
	public String getWeather() {
		
		return this.weather;
		
	}
	
	public void setTemperature(String scenTemperature) {
		
		this.temperature = scenTemperature;
		
	}
	
	public String getTemperature() {
		
		return this.temperature;
		
	}
	
	public void setGroundCondition(String scenGroundCondition) {
		
		this.groundCondition = scenGroundCondition;
		
	}
	
	public String getGroundCondition() {
		
		return this.groundCondition;
		
	}
	
	public void setEarlyIntel(String scenEarlyIntel) {
		
		this.earlyIntel = scenEarlyIntel;
		
	}
	
	public String getEarlyIntel() {
		
		return this.earlyIntel;
		
	}
	
	public void setMapWidth(String scenMapWidth) {
		
		this.mapWidth = scenMapWidth;
		mapWidthSet = true;
		
	}
	
	public String getMapWidth() {
		
		return this.mapWidth + " m";
		
	}
	
	public void setMapDepth(String scenMapDepth) {
		
		this.mapDepth = scenMapDepth;
		mapDepthSet = true;
		
	}
	
	public String getMapDepth() {
		
		return this.mapDepth + " m";
		
	}
	
	public void setMapArea() {
		
		if (mapWidthSet && mapDepthSet){
			
			double area = ( (Integer.valueOf(mapWidth) / 1000.0) * (Integer.valueOf(mapDepth) / 1000.0) );
			DecimalFormat format = new DecimalFormat("#.###");

			this.mapArea = String.valueOf( format.format(new Double(area)) );
			
		} else {
			
			this.mapArea = "0";
			
		}
		
	}
	
	public String getMapArea(){
		
		return (this.mapArea + " Sq. km");
		
	}
	
	public void setSizeModifier(String scenSizeModifier) {
		
		this.sizeModifier = scenSizeModifier;
		
	}
	
	public String getSizeModifier() {
		
		return this.sizeModifier;
		
	}
	
	public void setNumberOfModulesRequired(int numberOfModules) {
		
		this.numberOfModulesRequired = numberOfModules;
		
	}
	
	public int getNumberOfModulesRequired() {
		
		return this.numberOfModulesRequired;
		
	}
	
	public void setFirstModuleRequired(String requiredModule) {
		
		this.firstModuleRequired = requiredModule;
		
	}
	
	public String getFirstModuleRequired() {
		
		return this.firstModuleRequired;
		
	}
	
	public void setSecondModuleRequired(String requiredModule) {
		
		this.secondModuleRequired = requiredModule;
		
	}
	
	public String getSecondModuleRequired() {
		
		return this.secondModuleRequired;
		
	}
	
	public void setThirdModuleRequired(String requiredModule) {
		
		this.thirdModuleRequired = requiredModule;
		
	}
	
	public String getThirdModuleRequired() {
		
		return this.thirdModuleRequired;
		
	}
	
	public void setFourthModuleRequired(String requiredModule) {
		
		this.fourthModuleRequired = requiredModule;
		
	}
	
	public String getFourthModuleRequired() {
		
		return this.fourthModuleRequired;
		
	}
	
	public void setOptionalModulesRequired(String optionalModules) {
		
		this.optionalModulesRequired = optionalModules;
		
		optionalModulesDefined = true;
		
	}
	
	public String getOptionalModulesRequired() {
		
		return this.optionalModulesRequired;
		
	}
	
	public boolean getOptionalModulesDefined() {
		
		return this.optionalModulesDefined;
		
	}
	
	public void setModTags(String[] inputTags){
		
		this.modTags = inputTags;
		
	}
	
	public String[] getModTags(){
		
		return this.modTags;
		
	}
	
}
