/* 
    CMx2 ScAnCaDe - Scenario Analyser and Campaign Decompiler
    
    Analysing scenarios and campaigns and creating sortable listings 
    containing meta-information about the analysed scenarios/campaigns.
    Also, extract all scenarios belonging to a campaign from the
    campaign file, enabling stand-alone play of these battles.
    Programmed to support the players and scenario designers of the 
    Combat Mission - CMx2 engine WW2 computer games produced by
    Battlefront Inc. (www.battlefront.com)
    
    Copyright (C) 2014  Michael Wellhfer (michael@wellhoefer.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    Change History:
    
    ---------------------------------------------------------------------
    | VERSION |   DATE     |             DESCRIPTION                    |
    |---------|------------|--------------------------------------------|
    |   1.0   | 10/06/2013 | Initial release version.                   |
    |---------|------------|--------------------------------------------|
    |   1.1   | 15/10/2013 | Updated version for CMBN Market Garden,    |
    |         |            | which is v2.10.                            |
    |---------|------------|--------------------------------------------|
    |   1.2   | 17/11/2013 | Updated for CMBN v2.11 and CMFI v1.11.     |
    |---------|------------|--------------------------------------------|
    |   1.3   | 15/12/2013 | Updated for CMBN v2.12 and CMFI v1.12.     |
    |---------|------------|--------------------------------------------|
    |   1.4   | 22/02/2014 | Updated with further bugfixes and added    |
    |         |            | provision of required Modules info.        |
    |---------|------------|--------------------------------------------|
    |   1.5   | 25/02/2014 | Added improved exception logging (full     |
    |         |            | stack trace).                              |
    |---------|------------|--------------------------------------------|
    |   1.6   | 27/09/2014 | Updated for CMBN v2.20 and CMFI v1.20 and  |
    |         |            | some general streamlining/housekeeping.    |
    |---------|------------|--------------------------------------------|
    |   1.7   | 02/11/2014 | Updated for CMBN v3.10 and the Vehicle Pack|
    |---------|------------|--------------------------------------------|
    |   1.8   | 23/12/2014 | Updated for CMBN v3.11 and CMRT v1.01 to   |
    |         |            | v1.03 as well as optional modules for CMBN.|
    |---------|------------|--------------------------------------------|
    |   1.9   | 10/04/2016 | Updated for CMBN v3.12 and Battle Pack #1, |
    |         |            | as well as CMFB v1.00.                     |
    |---------|------------|--------------------------------------------|
*/

package Main_Application;

import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import Parser.CampaignReader;
import Parser.ScenarioReader;

public class CMx2_ScAn_CaDe_Main {
	
	private static final String versionString = "CMx2 ScAnCaDe v1.9";
	
	public static void main(String[] args) throws IOException {
	
		System.setProperty("org.graphstream.ui.renderer", "org.graphstream.ui.j2dviewer.J2DGraphRenderer");
		
		try{
			ScenarioReader scenReader = new ScenarioReader();
		
			if (scenReader.readScenariosfromCurrentDirectory()) {
	    
				scenReader.writeOutputtoCurrentDirectory();
	    	
			}
		
			CampaignReader campaignReader = new CampaignReader();
		
			if (campaignReader.readCampaignsfromCurrentDirectory()) {
	    
				campaignReader.writeOutputtoCurrentDirectory();
	    	
			}
		}
		
		catch (Exception e){
			
			FileWriter ExceptionOutputWriter = new FileWriter("exception.log");
			
			DateFormat df = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");

			Date today = Calendar.getInstance().getTime();        

			String reportDate = df.format(today);
			
			ExceptionOutputWriter.write(versionString + System.getProperty("line.separator"));
			ExceptionOutputWriter.write(reportDate + System.getProperty("line.separator") + System.getProperty("line.separator"));
			
			StackTraceElement stack[] = e.getStackTrace();

		    for (int i = 0; i < stack.length; i++) {
		    	
		        ExceptionOutputWriter.write(stack[i].toString() + System.getProperty("line.separator") + System.getProperty("line.separator"));
		        
		    }
		
			ExceptionOutputWriter.flush();
			ExceptionOutputWriter.close();
			
		}
	}

}

