/* 
    CMx2 ScAnCaDe - Scenario Analyser and Campaign Decompiler
    
    Analysing scenarios and campaigns and creating sortable listings 
    containing meta-information about the analysed scenarios/campaigns.
    Also, extract all scenarios belonging to a campaign from the
    campaign file, enabling stand-alone play of these battles.
    Programmed to support the players and scenario designers of the 
    Combat Mission - CMx2 engine WW2 computer games produced by
    Battlefront Inc. (www.battlefront.com)
    
    Copyright (C) 2014  Michael Wellhfer (michael@wellhoefer.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    Change History:
    
    ---------------------------------------------------------------------
    | VERSION |   DATE     |             DESCRIPTION                    |
    |---------|------------|--------------------------------------------|
    |   1.0   | 10/06/2013 | Initial release version.                   |
    |---------|------------|--------------------------------------------|
    |   1.1   | 15/10/2013 | Added writing of Region for scenarios(XML).|
    |---------|------------|--------------------------------------------|
    |   1.4   | 22/02/2014 | Added writing of Modules required for      |
    |         |            | scenarios and campaigns(XML).              |
    |---------|------------|--------------------------------------------|
    |   1.5   | 06/04/2014 | Added writing of mod tags for              |
    |         |            | scenarios(XML) and campaign tree picture   |
    |         |            | for campaigns(XML).                        |
    |---------|------------|--------------------------------------------|
    |   1.8   | 23/12/2014 | Added writing of optional Modules info.    |
    |---------|------------|--------------------------------------------|
    |  1.9#1  | 03/01/2016 | Removed DOCTYPE declaration from XML files.|
    |---------|------------|--------------------------------------------|
    |  1.9#2  | 27/02/2016 | Added writing of fourth module information.|
    |---------|------------|--------------------------------------------|
*/

package Output;

import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.Set;

import Data.CMx2_Campaign;
import Data.CMx2_CampaignDecisionBlock;
import Data.CMx2_Scenario;

public class OutputWriter {

	private String correctXMLOutput(String inputString){
		
		String returnString = inputString;
		String correctedString1 = "";
		String correctedString2 = "";
		String correctedString3 = "";
		String correctedString4 = "";
		String correctedString5 = "";
		
		if (inputString.contains("&")){
			
			correctedString1 = inputString.replace("&", "&amp;");
			returnString = correctedString1;
			
		} else {
			
			correctedString1 = inputString;
			
		}
		
		if (correctedString1.contains(">")){
			
			correctedString2 = correctedString1.replace(">", "&gt;");
			returnString = correctedString2;
			
		} else {
			
			correctedString2 = correctedString1;
			
		}
		
		if (correctedString2.contains("<")){
			
			correctedString3 = correctedString2.replace("<", "&lt;");
			returnString = correctedString3;
			
		} else {
			
			correctedString3 = correctedString2;
			
		}
		
		if (correctedString3.contains("'")){
			
			correctedString4 = correctedString3.replace("'", "&apos;");
			returnString = correctedString4;
			
		} else {
			
			correctedString4 = correctedString3;
			
		}
		
		if (correctedString4.contains("\"")){
			
			correctedString5 = correctedString4.replace("\"", "&quot;");
			returnString = correctedString5;
			
		}
		
		return returnString;
		
	}
	
	public void writeCampaignHTMLOutput(Set<CMx2_Campaign> inputCampaignList) throws IOException{
		
		FileWriter HTMLOutputWriter = null;
		
		try {
			
			HTMLOutputWriter = new FileWriter("Campaign_Listing.html");		
		
			HTMLOutputWriter.write("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"" + System.getProperty("line.separator"));
			HTMLOutputWriter.write("    \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">" + System.getProperty("line.separator"));
	
			HTMLOutputWriter.write("" + System.getProperty("line.separator"));
	
			HTMLOutputWriter.write("<html xmlns=\"http://www.w3.org/1999/xhtml\" lang=\"en\" xml:lang=\"en\">" + System.getProperty("line.separator"));
	
			HTMLOutputWriter.write("" + System.getProperty("line.separator"));
	
			HTMLOutputWriter.write("<head>" + System.getProperty("line.separator"));
	
			HTMLOutputWriter.write("" + System.getProperty("line.separator"));
		
			HTMLOutputWriter.write("<link href=\"tables_layout.css\" rel=\"stylesheet\" type=\"text/css\" />");
		
			HTMLOutputWriter.write("" + System.getProperty("line.separator"));
	
			HTMLOutputWriter.write("<title>Combat Mission - Battle for Normandy - Campaign Listing</title>" + System.getProperty("line.separator"));
	
			HTMLOutputWriter.write("" + System.getProperty("line.separator"));
	
			HTMLOutputWriter.write("<script src=\"sorttable.js\" type=\"text/javascript\"></script> " + System.getProperty("line.separator"));
	
			HTMLOutputWriter.write("" + System.getProperty("line.separator"));
	
			HTMLOutputWriter.write("</head>" + System.getProperty("line.separator"));
	
			HTMLOutputWriter.write("" + System.getProperty("line.separator"));
	
			HTMLOutputWriter.write("<body>" + System.getProperty("line.separator"));
	
			HTMLOutputWriter.write("" + System.getProperty("line.separator"));
		
			Iterator<CMx2_Campaign> campaignIterator = inputCampaignList.iterator();
		
			while (campaignIterator.hasNext()){
			
				CMx2_Campaign outputCampaign =((CMx2_Campaign) campaignIterator.next());
			
				HTMLOutputWriter.write("<h1>CMBN - Campaign Script Listing for Campaign \"" + outputCampaign.getTitle() + "\"" + "</h1>" + System.getProperty("line.separator"));
			
				HTMLOutputWriter.write("" + System.getProperty("line.separator"));
			
				HTMLOutputWriter.write("<img class=\"campaign\" src=\"campaignpics/" + outputCampaign.getfileName() + ".bmp\" alt=\"" + outputCampaign.getfileName() + "\">"+ System.getProperty("line.separator"));
			
				HTMLOutputWriter.write("" + System.getProperty("line.separator"));
			
				HTMLOutputWriter.write("<h2>Campaign created with CMBN game version " + outputCampaign.getCreatedwithVersion() + "</h2>" + System.getProperty("line.separator"));
			
				HTMLOutputWriter.write("<table id=\"main\" class=\"sortable\" style=\"table-layout:fixed\">" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("	<tr>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"title_col\">Battle Number &" + "<br>" + "Battle Name</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"winthreshold_col\">Win Threshold</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"nextbattle_col\">Next Battle after Win</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"nextbattle_col\">Next Battle after Defeat</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"victory_col\">Minimum Required Victory</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"victory_col\">Maximum Allowed Victory</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"repair_col\">Blue Refit</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"repair_col\">Blue Repair Vehicle</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"repair_col\">Blue Resupply</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"repair_col\">Blue Rest</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"repair_col\">Red Refit</th>" + System.getProperty("line.separator"));		
				HTMLOutputWriter.write("		<th class=\"repair_col\">Red Repair Vehicle</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"repair_col\">Red Resupply</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"repair_col\">Red Rest</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("	</tr>" + System.getProperty("line.separator"));
			
				for (int currentScenarioNumber = 0; currentScenarioNumber < outputCampaign.getNumberOfScenarios(); currentScenarioNumber++) {
				
					CMx2_CampaignDecisionBlock currentDecisionBlock = outputCampaign.getCMBNCampaignDecisionBlock(currentScenarioNumber);
				
					CMx2_Scenario currentScenario = outputCampaign.getCampaignScenario(currentScenarioNumber); 
				
					HTMLOutputWriter.write("	<tr>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"title\">" + currentScenario.getCampaignBattleNumber() + "<br>" + currentScenario.getTitle() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"winthreshold\">" + currentDecisionBlock.getwinThreshold() + "</td>" + System.getProperty("line.separator"));
				
					if (currentDecisionBlock.getnextBattleWin() == 65535){
					
						HTMLOutputWriter.write("		<td class=\"nextbattle\">Campaign End</td>" + System.getProperty("line.separator"));
					
					} else {
					
						String zeroPrefix = "#"; 
					
						if (currentDecisionBlock.getnextBattleWin() <= 9){
						
							zeroPrefix = "#0";
						
						}
					
						HTMLOutputWriter.write("		<td class=\"nextbattle\"> " + zeroPrefix + currentDecisionBlock.getnextBattleWin() + " </td>" + System.getProperty("line.separator"));
					
					}
				
					if (currentDecisionBlock.getnextBattleLose() == 65535){
					
						HTMLOutputWriter.write("		<td class=\"nextbattle\">Campaign End</td>" + System.getProperty("line.separator"));
					
					} else {
					
                    	String zeroPrefix = "#"; 
					
						if (currentDecisionBlock.getnextBattleWin() <= 9){
						
							zeroPrefix = "#0";
						
						}
					
						HTMLOutputWriter.write("		<td class=\"nextbattle\"> " + zeroPrefix + currentDecisionBlock.getnextBattleLose() + " </td>" + System.getProperty("line.separator"));
					
					}
				
					HTMLOutputWriter.write("		<td class=\"victory\">" + currentDecisionBlock.getminimumRequiredCampaignVictory() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"victory\">" + currentDecisionBlock.getmaximumAllowedCampaignVictory() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"repair\">" + currentDecisionBlock.getblueRefit() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"repair\">" + currentDecisionBlock.getblueRepairVehicle() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"repair\">" + currentDecisionBlock.getblueResupply() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"repair\">" + currentDecisionBlock.getblueRest() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"repair\">" + currentDecisionBlock.getredRefit() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"repair\">" + currentDecisionBlock.getredRepairVehicle() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"repair\">" + currentDecisionBlock.getredResupply() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"repair\">" + currentDecisionBlock.getredRest() + "</td>" + System.getProperty("line.separator"));
				
				}
				
				HTMLOutputWriter.write("</table>" + System.getProperty("line.separator"));
			
			}    

			HTMLOutputWriter.write("</body>" + System.getProperty("line.separator"));
			HTMLOutputWriter.write("</html>" + System.getProperty("line.separator"));		
		}
		
		finally{
			
			if (HTMLOutputWriter != null){
				
				HTMLOutputWriter.flush();
				HTMLOutputWriter.close();
				
			}
		}
	}
	
	public void writeCampaignXMLOutput(Set<CMx2_Campaign> inputCampaignList) throws IOException{
		
		FileWriter XMLOutputWriter = null;
		
		
		try{
			
			XMLOutputWriter = new FileWriter("Campaign_Listing.xml");
		
			XMLOutputWriter.write("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" + System.getProperty("line.separator"));
			//XMLOutputWriter.write("<!DOCTYPE Campaign_Listing SYSTEM \"Campaign_Listing.dtd\">" + System.getProperty("line.separator"));
			XMLOutputWriter.write("<?xml-stylesheet type=\"text/xsl\" href=\"Listing_Layout.xsl\" ?>" + System.getProperty("line.separator"));

			XMLOutputWriter.write("" + System.getProperty("line.separator"));
		
			XMLOutputWriter.write("<Campaign_Listing>" + System.getProperty("line.separator"));
		
			XMLOutputWriter.write("" + System.getProperty("line.separator"));
		
			XMLOutputWriter.write("   <Campaign_List>" + System.getProperty("line.separator"));
		
			XMLOutputWriter.write("" + System.getProperty("line.separator"));
		
			Iterator<CMx2_Campaign> campaignIterator = inputCampaignList.iterator();
		
			while (campaignIterator.hasNext()){
			
				CMx2_Campaign outputCampaign =((CMx2_Campaign) campaignIterator.next());
			
				XMLOutputWriter.write("      <Campaign>" + System.getProperty("line.separator"));
			
				XMLOutputWriter.write("" + System.getProperty("line.separator"));
			
				XMLOutputWriter.write("         <title>" + correctXMLOutput(outputCampaign.getTitle()) + "</title>" + System.getProperty("line.separator"));
				XMLOutputWriter.write("         <picture_link>" + "campaignpics/" + correctXMLOutput(outputCampaign.getfileName()) + ".bmp" + "</picture_link>" + System.getProperty("line.separator"));
				XMLOutputWriter.write("         <campaign_tree_link>" + correctXMLOutput(outputCampaign.getCampaignTreeFilename()) + "</campaign_tree_link>" + System.getProperty("line.separator"));
				XMLOutputWriter.write("         <game_version>" + outputCampaign.getCreatedwithVersion() + "</game_version>" + System.getProperty("line.separator"));
				
				XMLOutputWriter.write("         <Modules_Required>" + System.getProperty("line.separator"));
        		
        		switch (outputCampaign.getNumberOfModulesRequired()){
      		    	
        			case 0:
    				
        				if (outputCampaign.getOptionalModulesDefined()){
    					
        					XMLOutputWriter.write("         <module1>" + "[" + outputCampaign.getOptionalModulesRequired() + "]" + " </module1>" + System.getProperty("line.separator"));
    					
        				}
        				break;
        		
        		    case 1:
        		    	
        		    	XMLOutputWriter.write("            <module1>" + outputCampaign.getFirstModuleRequired() + " </module1>" + System.getProperty("line.separator"));
        		    	
        		    	if (outputCampaign.getOptionalModulesDefined()){
        					
        					XMLOutputWriter.write("         <module2>" + "[" + outputCampaign.getOptionalModulesRequired() + "]" + " </module2>" + System.getProperty("line.separator"));
        					
        				}
        		    	
        		    	break;
        		    	
        		    case 2:
        		    	
        		    	XMLOutputWriter.write("            <module1>" + outputCampaign.getFirstModuleRequired() + " </module1>" + System.getProperty("line.separator"));
        		    	XMLOutputWriter.write("            <module2>" + outputCampaign.getSecondModuleRequired() + " </module2>" + System.getProperty("line.separator"));
        		    	break;
        		    	
        		    case 3:
        		    	
        		    	XMLOutputWriter.write("            <module1>" + outputCampaign.getFirstModuleRequired() + " </module1>" + System.getProperty("line.separator"));
        		    	XMLOutputWriter.write("            <module2>" + outputCampaign.getSecondModuleRequired() + " </module2>" + System.getProperty("line.separator"));
        		    	XMLOutputWriter.write("            <module3>" + outputCampaign.getThirdModuleRequired() + " </module3>" + System.getProperty("line.separator"));
        		    	break;
        		    	
        		    case 4:
        		    	
        		    	XMLOutputWriter.write("            <module1>" + outputCampaign.getFirstModuleRequired() + " </module1>" + System.getProperty("line.separator"));
        		    	XMLOutputWriter.write("            <module2>" + outputCampaign.getSecondModuleRequired() + " </module2>" + System.getProperty("line.separator"));
        		    	XMLOutputWriter.write("            <module3>" + outputCampaign.getThirdModuleRequired() + " </module3>" + System.getProperty("line.separator"));
        		    	XMLOutputWriter.write("            <module4>" + outputCampaign.getFourthModuleRequired() + " </module4>" + System.getProperty("line.separator"));
        		    	break;
        		    	
        		    default:
        		    	
        		    	XMLOutputWriter.write("            <module1>None</module1>" + System.getProperty("line.separator"));
        		    	break;
        		
        		}
        		
        		XMLOutputWriter.write("         </Modules_Required>" + System.getProperty("line.separator"));
			
				XMLOutputWriter.write("" + System.getProperty("line.separator"));
			
				for (int currentScenarioNumber = 0; currentScenarioNumber < outputCampaign.getNumberOfScenarios(); currentScenarioNumber++) {
            	
					CMx2_CampaignDecisionBlock currentDecisionBlock = outputCampaign.getCMBNCampaignDecisionBlock(currentScenarioNumber);
				
					if (currentDecisionBlock.getcontainsScenario()) {
                
						CMx2_Scenario currentScenario = outputCampaign.getCampaignScenario(currentScenarioNumber);
				
						XMLOutputWriter.write("         <Campaign_Decision_Block>" + System.getProperty("line.separator"));
				
						XMLOutputWriter.write("            <Scenario_Title>" + System.getProperty("line.separator"));
				
						XMLOutputWriter.write("               <number_in_campaign>" + currentScenario.getCampaignBattleNumber() + "</number_in_campaign>" + System.getProperty("line.separator"));
				
						XMLOutputWriter.write("               <battle_title>" + correctXMLOutput(currentScenario.getTitle()) + "</battle_title>" + System.getProperty("line.separator"));
				
						XMLOutputWriter.write("            </Scenario_Title>" + System.getProperty("line.separator"));
				
						XMLOutputWriter.write("            <win_threshold>" + currentDecisionBlock.getwinThreshold() + "</win_threshold>" + System.getProperty("line.separator"));
				
						if (currentDecisionBlock.getnextBattleWin() == 65535){
					
							XMLOutputWriter.write("            <next_battle_win>Campaign End</next_battle_win>" + System.getProperty("line.separator"));
					
						} else {
					
							String zeroPrefix = "#"; 
					
							if (currentDecisionBlock.getnextBattleWin() <= 9){
						
								zeroPrefix = "#0";
						
							}
					
							XMLOutputWriter.write("            <next_battle_win>" + zeroPrefix + currentDecisionBlock.getnextBattleWin() + "</next_battle_win>" + System.getProperty("line.separator"));
					
						}
				
						if (currentDecisionBlock.getnextBattleLose() == 65535){
					
							XMLOutputWriter.write("            <next_battle_lose>Campaign End</next_battle_lose>" + System.getProperty("line.separator"));
					
						} else {
					
							String zeroPrefix = "#"; 
					
							if (currentDecisionBlock.getnextBattleWin() <= 9){
						
								zeroPrefix = "#0";
						
							}
					
							XMLOutputWriter.write("            <next_battle_lose>" + zeroPrefix + currentDecisionBlock.getnextBattleLose() + "</next_battle_lose>" + System.getProperty("line.separator"));
					
						}
				
						XMLOutputWriter.write("            <minimum_required_victory>" + currentDecisionBlock.getminimumRequiredCampaignVictory() + "</minimum_required_victory>" + System.getProperty("line.separator"));
						XMLOutputWriter.write("            <maximum_allowed_victory>" + currentDecisionBlock.getmaximumAllowedCampaignVictory() + "</maximum_allowed_victory>" + System.getProperty("line.separator"));
				
						XMLOutputWriter.write("            <blue_refit>" + currentDecisionBlock.getblueRefit() + "</blue_refit>" + System.getProperty("line.separator"));
						XMLOutputWriter.write("            <blue_repair_vehicle>" + currentDecisionBlock.getblueRepairVehicle() + "</blue_repair_vehicle>" + System.getProperty("line.separator"));
						XMLOutputWriter.write("            <blue_resupply>" + currentDecisionBlock.getblueResupply() + "</blue_resupply>" + System.getProperty("line.separator"));
						XMLOutputWriter.write("            <blue_rest>" + currentDecisionBlock.getblueRest() + "</blue_rest>" + System.getProperty("line.separator"));
				
						XMLOutputWriter.write("            <red_refit>" + currentDecisionBlock.getredRefit() + "</red_refit>" + System.getProperty("line.separator"));
						XMLOutputWriter.write("            <red_repair_vehicle>" + currentDecisionBlock.getredRepairVehicle() + "</red_repair_vehicle>" + System.getProperty("line.separator"));
						XMLOutputWriter.write("            <red_resupply>" + currentDecisionBlock.getredResupply() + "</red_resupply>" + System.getProperty("line.separator"));
						XMLOutputWriter.write("            <red_rest>" + currentDecisionBlock.getredRest() + "</red_rest>" + System.getProperty("line.separator"));
				
						XMLOutputWriter.write("         </Campaign_Decision_Block>" + System.getProperty("line.separator"));
				
						XMLOutputWriter.write("" + System.getProperty("line.separator"));
            
					}
                	
				}
			
				XMLOutputWriter.write("      </Campaign>" + System.getProperty("line.separator"));
			
			}
		
			XMLOutputWriter.write("   </Campaign_List>" + System.getProperty("line.separator"));
		
			XMLOutputWriter.write("</Campaign_Listing>");
			
		}
		finally{
			
			if (XMLOutputWriter != null){
				
				XMLOutputWriter.flush();
				XMLOutputWriter.close();
				
			}
			
		}
		
	}
	
	public void writeScenarioHTMLOutput(Set<CMx2_Scenario> inputScenarioList, boolean isCampaignOutput, String campaignDirPrefix, String campaignFileName, String campaignTitle) throws IOException{
		
		if (!inputScenarioList.isEmpty()) {
			
			FileWriter HTMLOutputWriter = null;
			
			try{
			
				if (isCampaignOutput){
				
					HTMLOutputWriter = new FileWriter(campaignDirPrefix + campaignFileName + " - Scenario_Listing.html");
				
				}else{
				
					HTMLOutputWriter = new FileWriter(campaignDirPrefix + "Scenario_Listing.html");
				
				}
			
		
				HTMLOutputWriter.write("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("    \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">" + System.getProperty("line.separator"));
		
				HTMLOutputWriter.write("" + System.getProperty("line.separator"));
		
				HTMLOutputWriter.write("<html xmlns=\"http://www.w3.org/1999/xhtml\" lang=\"en\" xml:lang=\"en\">" + System.getProperty("line.separator"));
		
				HTMLOutputWriter.write("" + System.getProperty("line.separator"));
		
				HTMLOutputWriter.write("<head>" + System.getProperty("line.separator"));
		
				HTMLOutputWriter.write("" + System.getProperty("line.separator"));
			
				HTMLOutputWriter.write("<link href=\"tables_layout.css\" rel=\"stylesheet\" type=\"text/css\" />");
			
				HTMLOutputWriter.write("" + System.getProperty("line.separator"));
		
				if (isCampaignOutput){
				
					HTMLOutputWriter.write("<title>CMBN - Scenario Listing for Campaign \"" + campaignTitle + "\"" + "</title>" + System.getProperty("line.separator"));
				
				} else {
				
					HTMLOutputWriter.write("<title>Combat Mission - Battle for Normandy - Scenario Listing</title>" + System.getProperty("line.separator"));
				
				}
		
				HTMLOutputWriter.write("" + System.getProperty("line.separator"));
		
				HTMLOutputWriter.write("<script src=\"sorttable.js\" type=\"text/javascript\"></script> " + System.getProperty("line.separator"));
		
				HTMLOutputWriter.write("" + System.getProperty("line.separator"));
		
				HTMLOutputWriter.write("</head>" + System.getProperty("line.separator"));
		
				HTMLOutputWriter.write("" + System.getProperty("line.separator"));
		
				HTMLOutputWriter.write("<body>" + System.getProperty("line.separator"));
		
				HTMLOutputWriter.write("" + System.getProperty("line.separator"));
		
				if (isCampaignOutput){
				
					HTMLOutputWriter.write("<h1>CMBN - Scenario Listing for Campaign \"" + campaignTitle + "\"" + "</h1>" + System.getProperty("line.separator"));
				
				} else {
				
					HTMLOutputWriter.write("<h1>Combat Mission - Battle for Normandy - Scenario Listing</h1>" + System.getProperty("line.separator"));
				
				}
			
			
		
				HTMLOutputWriter.write("" + System.getProperty("line.separator"));
		
				HTMLOutputWriter.write("<table id=\"main\" class=\"sortable\" style=\"table-layout:fixed\">" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("	<tr>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"title_col\">Title</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"picture_col\">Picture</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"battletype_col\">Battle Type</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"description_col\">Description</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"date_col\">Date (dd/mm/yyyy)</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"timeofday_col\">Time of Day</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"partofday_col\">Part of the Day</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"duration_col\">Duration</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"battlesize_col\">Battle Size</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"mapsize_col\">Map Size (Area - Width/Depth)</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"environment_col\">Environment</th>" + System.getProperty("line.separator"));		
				HTMLOutputWriter.write("		<th class=\"weather_col\">Weather</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"temperature_col\">Temperature</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"groundcondition_col\">Ground Condition</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"earlyintel_col\">Early Intel</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"gameversion_col\">Game Version</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("		<th class=\"sizemodifier_col\">The Blitz Size Modifier</th>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("	</tr>" + System.getProperty("line.separator"));
		
				Iterator<CMx2_Scenario> scenarioIterator = inputScenarioList.iterator(); 
	    
				while (scenarioIterator.hasNext()) {
	    	
					CMx2_Scenario outputScenario = ((CMx2_Scenario) scenarioIterator.next());
	    	
					HTMLOutputWriter.write("	<tr>" + System.getProperty("line.separator"));
				
					if (isCampaignOutput){
					
						HTMLOutputWriter.write("		<td class=\"title\">" + outputScenario.getCampaignBattleNumber() + "<br>" + outputScenario.getTitle() + "</td>" + System.getProperty("line.separator"));
					
					} else {
					
						HTMLOutputWriter.write("		<td class=\"title\">" + outputScenario.getTitle() + "</td>" + System.getProperty("line.separator"));
					
					}
				
					HTMLOutputWriter.write("		<td class=\"picture\">" + "<img class=\"scenario\" src=\"scenpics/" + outputScenario.getbitmapFileName() + ".bmp\" alt=\"" + outputScenario.getTitle() + "\">" + "</th>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"battletype\">" + outputScenario.getBattleType() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"description\">" + outputScenario.getDescription()+ "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"date\">" + outputScenario.getDay() + "/" + outputScenario.getMonth() + "/" + outputScenario.getYear() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"timeofday\">" + outputScenario.getTime() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"partofday\">" + outputScenario.getDaylight() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"duration\">" + outputScenario.getBattleLength()+ "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"battlesize\">" + outputScenario.getBattleSize() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"mapsize\">" + outputScenario.getMapArea() + "<br>" + "(" + outputScenario.getMapWidth() + " / " + outputScenario.getMapDepth() + ")" + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"environment\">" + outputScenario.getEnvironment() + "</td>" + System.getProperty("line.separator"));		
					HTMLOutputWriter.write("		<td class=\"weather\">" + outputScenario.getWeather() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"temperature\">" + outputScenario.getTemperature() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"groundcondition\">" + outputScenario.getGroundCondition() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"earlyintel\">" + outputScenario.getEarlyIntel() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"gameversion\">" + outputScenario.getCreatedwithVersion() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("		<td class=\"sizemodifier\">" + outputScenario.getSizeModifier() + "</td>" + System.getProperty("line.separator"));
					HTMLOutputWriter.write("	</tr>" + System.getProperty("line.separator"));
	    	
				}
	    
				HTMLOutputWriter.write("</table>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("</body>" + System.getProperty("line.separator"));
				HTMLOutputWriter.write("</html>" + System.getProperty("line.separator"));
				
			}
			finally{
				
				if (HTMLOutputWriter != null){
					
					HTMLOutputWriter.flush();
					HTMLOutputWriter.close();
					
				}
				
			}
	    		
		}
		
	}
	
	public void writeScenarioXMLOutput(Set<CMx2_Scenario> inputScenarioList, boolean isCampaignOutput, String campaignDirPrefix, String campaignFileName, String campaignTitle) throws IOException{
		
		if (!inputScenarioList.isEmpty()) {
			
            FileWriter XMLOutputWriter = null;
            
            try{
			
            	if (isCampaignOutput){
				
            		XMLOutputWriter = new FileWriter(campaignDirPrefix + campaignFileName + " - Scenario_Listing.xml");
				
            	}else{
				
            		XMLOutputWriter = new FileWriter(campaignDirPrefix + "Scenario_Listing.xml");
				
            	}
			
            	XMLOutputWriter.write("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" + System.getProperty("line.separator"));
            	//XMLOutputWriter.write("<!DOCTYPE Scenario_Listing SYSTEM \"Scenario_Listing.dtd\">" + System.getProperty("line.separator"));
            	XMLOutputWriter.write("<?xml-stylesheet type=\"text/xsl\" href=\"Listing_Layout.xsl\" ?>" + System.getProperty("line.separator"));

            	XMLOutputWriter.write("" + System.getProperty("line.separator"));
			
            	XMLOutputWriter.write("<Scenario_Listing>" + System.getProperty("line.separator"));
			
            	XMLOutputWriter.write("" + System.getProperty("line.separator"));
			
            	XMLOutputWriter.write("   <Scenario_List>" + System.getProperty("line.separator"));
			
            	XMLOutputWriter.write("" + System.getProperty("line.separator"));
			
            	if (isCampaignOutput){
				
            		XMLOutputWriter.write("   <containing_campaign>" + correctXMLOutput(campaignTitle) + "</containing_campaign>" + System.getProperty("line.separator"));
				
            		XMLOutputWriter.write("" + System.getProperty("line.separator"));
				
            	}
			
            	Iterator<CMx2_Scenario> scenarioIterator = inputScenarioList.iterator(); 
		    
            	while (scenarioIterator.hasNext()) {
				
            		CMx2_Scenario outputScenario = ((CMx2_Scenario) scenarioIterator.next());
				
            		XMLOutputWriter.write("   <Scenario>" + System.getProperty("line.separator"));
				
            		XMLOutputWriter.write("      <Title>" + System.getProperty("line.separator"));
				
            		if (isCampaignOutput){
					
            			XMLOutputWriter.write("         <number_in_campaign>" + outputScenario.getCampaignBattleNumber() + "</number_in_campaign>" + System.getProperty("line.separator"));
					
            		}
				
            		if (outputScenario.getfileName().contains("QB")) {
					
            			XMLOutputWriter.write("         <battle_title>"  + correctXMLOutput(outputScenario.getTitle()) + "</battle_title>" + System.getProperty("line.separator"));
            			XMLOutputWriter.write("         <qb_filename>" + correctXMLOutput(outputScenario.getfileName()) + ".btt</qb_filename>" + System.getProperty("line.separator"));
					
            		} else {
					
            			XMLOutputWriter.write("         <battle_title>"  + correctXMLOutput(outputScenario.getTitle()) + "</battle_title>" + System.getProperty("line.separator"));
					
            		}
								
            		XMLOutputWriter.write("      </Title>" +System.getProperty("line.separator"));
				
            		XMLOutputWriter.write("      <picture_link>" + "scenpics/" + correctXMLOutput(outputScenario.getbitmapFileName()) + ".bmp" + "</picture_link>" + System.getProperty("line.separator"));
            		XMLOutputWriter.write("      <battle_type>" + outputScenario.getBattleType() + "</battle_type>" + System.getProperty("line.separator"));				
            		XMLOutputWriter.write("      <description>" + correctXMLOutput(outputScenario.getDescription()) + "</description>" + System.getProperty("line.separator"));				
            		XMLOutputWriter.write("      <date>" + outputScenario.getDay() + "/" + outputScenario.getMonth() + "/" + outputScenario.getYear() + "</date>" + System.getProperty("line.separator"));				
            		XMLOutputWriter.write("      <time_of_day>" + outputScenario.getTime() + "</time_of_day>" + System.getProperty("line.separator"));				
            		XMLOutputWriter.write("      <part_of_day>" + outputScenario.getDaylight() + "</part_of_day>" + System.getProperty("line.separator"));				
            		XMLOutputWriter.write("      <length>" + outputScenario.getBattleLength() + "</length>" + System.getProperty("line.separator"));				
            		XMLOutputWriter.write("      <battle_size>" + outputScenario.getBattleSize() + "</battle_size>" + System.getProperty("line.separator"));				
				
            		XMLOutputWriter.write("      <Map_Size>" + System.getProperty("line.separator"));				
            		XMLOutputWriter.write("         <map_area>" + outputScenario.getMapArea() + "</map_area>" + System.getProperty("line.separator"));
            		XMLOutputWriter.write("         <map_width>" + outputScenario.getMapWidth() + "</map_width>" + System.getProperty("line.separator"));
            		XMLOutputWriter.write("         <map_depth>" + outputScenario.getMapDepth() + "</map_depth>" + System.getProperty("line.separator"));
            		XMLOutputWriter.write("      </Map_Size>" + System.getProperty("line.separator"));
				    
            		XMLOutputWriter.write("      <region>" + outputScenario.getRegion() + "</region>" + System.getProperty("line.separator"));
            		XMLOutputWriter.write("      <environment>" + outputScenario.getEnvironment() + "</environment>" + System.getProperty("line.separator"));				
            		XMLOutputWriter.write("      <weather>" + outputScenario.getWeather() + "</weather>" + System.getProperty("line.separator"));				
            		XMLOutputWriter.write("      <temperature>" + outputScenario.getTemperature() + "</temperature>" + System.getProperty("line.separator"));				
            		XMLOutputWriter.write("      <ground_condition>" + outputScenario.getGroundCondition() + "</ground_condition>" + System.getProperty("line.separator"));				
            		XMLOutputWriter.write("      <early_intel>" + outputScenario.getEarlyIntel() + "</early_intel>" + System.getProperty("line.separator"));				
            		XMLOutputWriter.write("      <game_version>" + outputScenario.getCreatedwithVersion() + "</game_version>" + System.getProperty("line.separator"));
            		
            		XMLOutputWriter.write("      <Modules_Required>" + System.getProperty("line.separator"));
            		
            		switch (outputScenario.getNumberOfModulesRequired()){
          		    	
            			case 0:
            				
            				if (outputScenario.getOptionalModulesDefined()){
            					
            					XMLOutputWriter.write("         <module1>" + outputScenario.getOptionalModulesRequired() + "</module1>" + System.getProperty("line.separator"));
            					
            				}
            				break;
            		
            		    case 1:
            		    	
            		    	XMLOutputWriter.write("         <module1>" + outputScenario.getFirstModuleRequired() + "</module1>" + System.getProperty("line.separator"));
            		    	
            		    	if (outputScenario.getOptionalModulesDefined()){
            					
            					XMLOutputWriter.write("         <module2>" + outputScenario.getOptionalModulesRequired() + "</module2>" + System.getProperty("line.separator"));
            					
            				}
            		    	
            		    	break;
            		    	
            		    case 2:
            		    	
            		    	XMLOutputWriter.write("         <module1>" + outputScenario.getFirstModuleRequired() + "</module1>" + System.getProperty("line.separator"));
            		    	XMLOutputWriter.write("         <module2>" + outputScenario.getSecondModuleRequired() + "</module2>" + System.getProperty("line.separator"));
            		    	break;
            		    	
            		    case 3:
            		    	
            		    	XMLOutputWriter.write("         <module1>" + outputScenario.getFirstModuleRequired() + "</module1>" + System.getProperty("line.separator"));
            		    	XMLOutputWriter.write("         <module2>" + outputScenario.getSecondModuleRequired() + "</module2>" + System.getProperty("line.separator"));
            		    	XMLOutputWriter.write("         <module3>" + outputScenario.getThirdModuleRequired() + "</module3>" + System.getProperty("line.separator"));
            		    	break;
            		    	
            		    case 4:
            		    	
            		    	XMLOutputWriter.write("         <module1>" + outputScenario.getFirstModuleRequired() + "</module1>" + System.getProperty("line.separator"));
            		    	XMLOutputWriter.write("         <module2>" + outputScenario.getSecondModuleRequired() + "</module2>" + System.getProperty("line.separator"));
            		    	XMLOutputWriter.write("         <module3>" + outputScenario.getThirdModuleRequired() + "</module3>" + System.getProperty("line.separator"));
            		    	XMLOutputWriter.write("         <module4>" + outputScenario.getFourthModuleRequired() + "</module4>" + System.getProperty("line.separator"));
            		    	break;
            		    	
            		    default:
            		    	
            		    	XMLOutputWriter.write("         <module1>None</module1>" + System.getProperty("line.separator"));
            		    	break;
            		
            		}
            		
            		XMLOutputWriter.write("      </Modules_Required>" + System.getProperty("line.separator"));
            		
            		XMLOutputWriter.write("      <Mod_Tags>" + System.getProperty("line.separator"));
            		
            		if (outputScenario.getModTags() != null) {
            			
            			String[] modTags = outputScenario.getModTags();
            			
            			for (int currentTagIndex = 0; currentTagIndex < modTags.length; currentTagIndex++){
            				
            				XMLOutputWriter.write("         <mod_tag>" + correctXMLOutput(modTags[currentTagIndex]) + "</mod_tag>" + System.getProperty("line.separator"));
            				
            			}
            			
            		}
            		
            		XMLOutputWriter.write("      </Mod_Tags>" + System.getProperty("line.separator"));
            		
            		XMLOutputWriter.write("      <size_modifier>" + outputScenario.getSizeModifier() + "</size_modifier>" + System.getProperty("line.separator"));
				
            		XMLOutputWriter.write("   </Scenario>" + System.getProperty("line.separator"));
				
            		XMLOutputWriter.write("" + System.getProperty("line.separator"));
				
            	}
	    	
            	XMLOutputWriter.write("   </Scenario_List>" + System.getProperty("line.separator"));
			
            	XMLOutputWriter.write("</Scenario_Listing>");
			
            }
            finally{
            	
            	if (XMLOutputWriter != null){
            		
        			XMLOutputWriter.flush();
        			XMLOutputWriter.close();
            		
            	}
            	
            }
	
		}
		
	}
	
}
