/* 
    CMx2 ScAnCaDe - Scenario Analyser and Campaign Decompiler
    
    Analysing scenarios and campaigns and creating sortable listings 
    containing meta-information about the analysed scenarios/campaigns.
    Also, extract all scenarios belonging to a campaign from the
    campaign file, enabling stand-alone play of these battles.
    Programmed to support the players and scenario designers of the 
    Combat Mission - CMx2 engine WW2 computer games produced by
    Battlefront Inc. (www.battlefront.com)
    
    Copyright (C) 2014  Michael Wellhfer (michael@wellhoefer.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    Change History:
    
    ---------------------------------------------------------------------
    | VERSION |   DATE     |             DESCRIPTION                    |
    |---------|------------|--------------------------------------------|
    |   1.0   | 10/06/2013 | Initial release version.                   |
    |---------|------------|--------------------------------------------|
    |   1.1   | 13/10/2013 | Added version CMBN v2.10.                  |
    |         |            | Also added analysis of Region parameter.   |
    |---------|------------|--------------------------------------------|
    |   1.2   | 17/11/2013 | Added version CMBN v2.11 and CMFI v1.11.   |
    |         |            | For this to work, added analysis of family |
    |         |            | parameter in analyse_and_return_Version.   |
    |---------|------------|--------------------------------------------|
    |  1.3#1  | 26/11/2013 | Replaced v2.11 with v2.12 for CMBN and     |
    |         |            | v1.11 with v1.12 for CMFI.                 |
    |---------|------------|--------------------------------------------|
    |  1.3#2  | 15/12/2013 | Added analysis of Year parameter.          |
    |---------|------------|--------------------------------------------|
    |  1.4#1  | 26/12/2013 | Updated analyse_and_return_Region to handle|
    |         |            | the same version for different families.   |
    |---------|------------|--------------------------------------------|
    |  1.4#2  | 09/02/2014 | Added analyse_and_return_Module_Required.  |
    |---------|------------|--------------------------------------------|
    |   1.5   | 05/04/2014 | Added version CMRT v1.00.                  |
    |---------|------------|--------------------------------------------|
    |   1.6   | 15/08/2014 | Added versions CMBN v2.20 and CMFI v1.20.  |
    |---------|------------|--------------------------------------------|
    |   1.7   | 02/11/2014 | Added version CMBN v3.10 and the Vehicle   |
    |         |            | Pack.                                      |
    |---------|------------|--------------------------------------------|
    |   1.8   | 23/12/2014 | Added versions CMRT v1.01 to v1.03 and     |
    |         |            | CMBN v3.11.                                |
    |---------|------------|--------------------------------------------|
    |  1.9#1  | 16/01/2016 | Added two CMFI v1.10 pre-release versions  |
    |         |            |(internal 119 & 120).                       |
    |---------|------------|--------------------------------------------|
    |  1.9#2  | 27/02/2016 | Added version CMBN v3.12 and the Battle    |
    |         |            | Pack #1.                                   |
    |---------|------------|--------------------------------------------|
    |  1.9#3  | 10/04/2016 | Added version CMFB v1.00 (and pre-releases)|
    |---------|------------|--------------------------------------------|
*/

package Parser;

public class ParameterAnalyser {
	
	private static String[][] SMTable = {{"1", "2", "3", "3", "4"},
										 {"2", "3", "3", "4", "5"},
										 {"4", "5", "6", "6", "7"},
										 {"6", "7", "7", "8", "9"},
										 {"8", "8", "9", "10", "10"}};
	
	public String analyse_and_return_Module_Required(int requiredModule, int gameFamilyType){
		
		String returnString ="";
		
		switch (gameFamilyType) {
		
		    //CMBN
			case 4: switch (requiredModule){
		
			           case 1: returnString = "Commonwealth"; break;
			           case 2: returnString = "Market Garden"; break;
			           case 3: returnString = "Vehicle Pack"; break; 
			           case 4: returnString = "Battle Pack #1"; break; 
			           default: returnString = "Unknown"; break;
			
			} break;
			
			//CMFI
			case 6: switch (requiredModule){
			
			           case 1:  returnString = "Gustav Line"; break;
	                   default: returnString = "Unknown"; break;
			
			} break;
			
			//CMRT
			case 8: switch (requiredModule){
			
	                   default: returnString = "Unknown"; break;
			
			} break;
			
			//CMFB
			case 12: switch (requiredModule){
			
	                   default: returnString = "Unknown"; break;
			
			} break;
			
			default: returnString = "Unknown"; break;
		
		}
		
		return returnString;
		
	}
	
	public String analyse_and_return_Version(int gameFamilyType, int engineVersionType){
		
		String returnString ="";
		
		switch (gameFamilyType) {
		    
		    //CMBN
			case 4: switch (engineVersionType){
			
						case 99:  returnString = "pre-release CMBN v1.00";   break; 		
						case 100: returnString = "CMBN v1.00";   break;
						case 103: returnString = "CMBN v1.01";   break;
						case 107: returnString = "CMBN v1.10";   break;
						case 117: returnString = "CMBN v2.00";   break;
						case 119: returnString = "CMBN v2.01";	 break;
						case 125: returnString = "CMBN v2.10";   break;
						case 126: returnString = "CMBN v2.12";   break;
						case 131: returnString = "CMBN v2.20";   break;
						case 135: returnString = "CMBN v3.10 to v3.12";   break;
						default:  returnString = "Unknown"; break;
			
			} break;
			
			//CMFI
			case 6: switch (engineVersionType){
			
						case 114: returnString = "CMFI v1.00";   break;
						case 115: returnString = "CMFI v1.01";   break;
						case 119: returnString = "pre-release CMFI v1.10";   break;
						case 120: returnString = "pre-release CMFI v1.10";   break;
						case 121: returnString = "pre-release CMFI v1.10";   break;
						case 122: returnString = "CMFI v1.10";   break;
						case 126: returnString = "CMFI v1.12";   break;
						case 131: returnString = "CMFI v1.20";   break;
						default:  returnString = "Unknown"; break;
						
			} break;
			
			//CMRT
			case 8: switch (engineVersionType){
			
						case 130: returnString = "pre-release CMRT v1.00";   break; 
						case 131: returnString = "CMRT v1.00 to v1.03";   break;
						default:  returnString = "Unknown"; break;
						
			} break;
			
			//CMFB
			case 12: switch (engineVersionType){
			
						case 141: returnString = "pre-release CMFB v1.00";   break; 
						case 142: returnString = "pre-release CMFB v1.00";   break; 
						case 143: returnString = "CMFB v1.00";   break; 
						default:  returnString = "Unknown"; break;
						
			} break;
				
			default: returnString = "Unknown"; break;
		
		}
	
		return returnString;
		
	}
	
	public String analyse_and_return_Region(int regionType, int gameFamilyType, int engineVersionType){
		
		String returnString ="";
		
		switch (gameFamilyType) {
		
			// CMBN
			case 4: switch (engineVersionType){
			
				case 99: case 100: case 103: case 107: case 117: case 119: 
				
					//CMBN before Market Garden (only France possible)
				
					if(regionType == 0){
					
						returnString = "France";
					 		
					}else{
					
						returnString = "Unknown";
					
					}
					
					break;
				
				case 125: case 126: case 131: case 135: 
					
					//CMBN after Market Garden (France - 0 , Holland - 1 possible)
					
					if(regionType == 0){
						
						returnString = "France";
						 		
					}else if(regionType == 1){
						
						returnString = "Holland";
						
					}else{
						
						returnString = "Unknown";
						
					}
						
					break;
			
			} break;
			
			// CMFI
			case 6: switch (engineVersionType){
			
				case 114: case 115: case 121: case 122: case 126: case 131:
				
					//CMFI including GL (only Italy possible)
				
					if(regionType == 0){
					
						returnString = "Italy";
					 		
					}else{
					
						returnString = "Unknown";
					
					}
					
					break;
			
			} break;
			
			// CMRT
			case 8: switch (engineVersionType){
						
				case 130: case 131: 
				
					//CMRT at release (Soviet Union - 0 , Eastern Europe - 1)
				
					if(regionType == 0){
					
						returnString = "Soviet Union";
					 		
					}else if(regionType == 1){
					
						returnString = "Eastern Europe";
					
					}else{
					
						returnString = "Unknown";
					
					}
					
					break;
					
			}break;
			
			// CMFB
			case 12: switch (engineVersionType){
									
				case 141: case 142: case 143: 
							
					//CMFB at release (Ardennes - 0 , France - 1, Germany - 2, Holland - 3)
							
					if(regionType == 0){
								
						returnString = "Ardennes";
								 		
					}else if(regionType == 1){
								
						returnString = "France";
								
					}else if(regionType == 2){
						
						returnString = "Germany";
						
					}else if(regionType == 3){
						
						returnString = "Holland";
						
					}else{
								
						returnString = "Unknown";
								
					}
								
					break;
								
			}break;
			
			default: returnString = "Unknown"; break;
		
		}
	
		return returnString;
		
	}
	
	public String analyse_and_return_Environment(byte environmentType){
		
		String returnString ="";
		
		switch (environmentType){
		
			case 0:	 returnString = "City"; 				break;
			case 1:  returnString = "Town"; 				break;
			case 2:  returnString = "Village"; 				break;
			case 3:  returnString = "Open"; 				break;
			case 4:  returnString = "Rough"; 				break;
			case 5:  returnString = "Forest";				break;
			case 6:  returnString = "Hills"; 				break;
			default: returnString = "Invalid Environment"; 	break;
		
		}
	
		return returnString;
		
	}
	
	public String analyse_and_return_Battle_Type(byte battletypeType, int gameFamilyType){
		
		String returnString ="";
		
		switch (gameFamilyType) {
		
			// CMBN & CMFI & CMFB
			case 4: case 6: case 12:
				
				switch (battletypeType){
				
				case 0:	 returnString = "Axis Assault";			break;
				case 1:  returnString = "Axis Attack";			break;
				case 2:  returnString = "Axis Probe";			break;
				case 3:  returnString = "Meeting Engagement";	break;
				case 4:  returnString = "Allied Probe";			break;
				case 5:  returnString = "Allied Attack";		break;
				case 6:  returnString = "Allied Assault";		break;
				default: returnString = "Invalid Battle Type"; 	break;
			
				} break;
				
			// CMRT
			case 8:
				
				switch (battletypeType){
				
				case 0:	 returnString = "Allied Assault";		break;
				case 1:  returnString = "Allied Attack";		break;
				case 2:  returnString = "Allied Probe";			break;
				case 3:  returnString = "Meeting Engagement";	break;
				case 4:  returnString = "Axis Probe";			break;
				case 5:  returnString = "Axis Attack";			break;
				case 6:  returnString = "Axis Assault";			break;
				default: returnString = "Invalid Battle Type"; 	break;
			
				} break;
				
			default: returnString = "Invalid Battle Type"; 	break;
		
		}
	
		return returnString;
		
	}
	
	public String analyse_and_return_Daylight(byte daylightType){
		
		String returnString ="";
		
		switch (daylightType){
		
			case 0:	 returnString = "Dawn";				break;
			case 1:  returnString = "Day";				break;
			case 2:  returnString = "Dusk";				break;
			case 3:  returnString = "Night";			break;
			default: returnString = "Invalid Daylight"; break;
		
		}
	
		return returnString;
		
	}
	
	public String analyse_and_return_Battle_Size(byte battlesizeType){
		
		String returnString ="";
		
		switch (battlesizeType){
		
			case 0:	 returnString = "Tiny";					break;
			case 1:  returnString = "Small";				break;
			case 2:  returnString = "Medium";				break;
			case 3:  returnString = "Large";				break;
			case 4:  returnString = "Huge";					break;
			default: returnString = "Invalid Battle Size"; 	break;
		
		}
	
		return returnString;
		
	}
	
	public String analyse_and_return_Year(int yearType){
		
		String returnString ="";
		
		switch (yearType){
		
		case 1943:	returnString = "1943";		break;
		case 1944:	returnString = "1944";		break;
		case 1945:	returnString = "1945";		break;
		default:  returnString = "Invalid Year"; 		break;
		
		}
		
		return returnString;
		
	}
	
	public String analyse_and_return_Month(byte monthType){
		
		String returnString ="";
		
		switch (monthType){
		
			case 0:	  returnString = "01";		break;
			case 1:   returnString = "02";		break;
			case 2:   returnString = "03";		break;
			case 3:   returnString = "04";		break;
			case 4:   returnString = "05";		break;
			case 5:   returnString = "06";		break;
			case 6:   returnString = "07";		break;
			case 7:   returnString = "08";		break;
			case 8:   returnString = "09";		break;
			case 9:   returnString = "10";		break;
			case 10:  returnString = "11";		break;
			case 11:  returnString = "12";		break;
			default:  returnString = "Invalid Month"; 		break;
		
		}
	
		return returnString;
		
	}
	
	public String analyse_and_return_Day(byte dayType){
		
		String returnString ="";
		
		switch (dayType){
		
			case 1:  returnString = "0" + Byte.toString(dayType);	break;
			case 2:  returnString = "0" + Byte.toString(dayType);	break;
			case 3:  returnString = "0" + Byte.toString(dayType);	break;
			case 4:  returnString = "0" + Byte.toString(dayType);	break;
			case 5:  returnString = "0" + Byte.toString(dayType);	break;
			case 6:  returnString = "0" + Byte.toString(dayType);	break;
			case 7:  returnString = "0" + Byte.toString(dayType);	break;
			case 8:  returnString = "0" + Byte.toString(dayType);	break;
			case 9:  returnString = "0" + Byte.toString(dayType);	break;
			case 10: returnString = Byte.toString(dayType);			break;
			case 11: returnString = Byte.toString(dayType);			break;
			case 12: returnString = Byte.toString(dayType);			break;
			case 13: returnString = Byte.toString(dayType);			break;
			case 14: returnString = Byte.toString(dayType);			break;
			case 15: returnString = Byte.toString(dayType);			break;
			case 16: returnString = Byte.toString(dayType);			break;
			case 17: returnString = Byte.toString(dayType);			break;
			case 18: returnString = Byte.toString(dayType);			break;
			case 19: returnString = Byte.toString(dayType);			break;
			case 20: returnString = Byte.toString(dayType);			break;
			case 21: returnString = Byte.toString(dayType);			break;
			case 22: returnString = Byte.toString(dayType);			break;
			case 23: returnString = Byte.toString(dayType);			break;
			case 24: returnString = Byte.toString(dayType);			break;
			case 25: returnString = Byte.toString(dayType);			break;
			case 26: returnString = Byte.toString(dayType);			break;
			case 27: returnString = Byte.toString(dayType);			break;
			case 28: returnString = Byte.toString(dayType);			break;
			case 29: returnString = Byte.toString(dayType);			break;
			case 30: returnString = Byte.toString(dayType);			break;
			case 31: returnString = Byte.toString(dayType);			break;
			default: returnString = "Invalid Day";	 				break;
		
		}
	
		return returnString;
		
	}
	
	public String analyse_and_return_TimeofDay_or_BattleLength(int timeinMinutes) {
		
		int hours = timeinMinutes / 60;
		int minutes = timeinMinutes % 60;
		
		String returnHours;
		String returnMinutes;
		
		if (hours < 10) {
			
			returnHours = "0" + String.valueOf(hours);
			
		} else {
			
			returnHours = String.valueOf(hours);
			
		}
		
		if (minutes < 10) {
			
			returnMinutes = "0" + String.valueOf(minutes);
			
		} else {
			
			returnMinutes = String.valueOf(minutes);
			
		}
		
		return (returnHours + ":" + returnMinutes);
		
	}
	
	public String analyse_and_return_Weather(byte weatherType){
		
		String returnString ="";
		
		switch (weatherType){
		
			case 0:	  returnString = "Clear";				break;
			case 1:   returnString = "Hazy";				break;
			case 2:   returnString = "Thick Haze";			break;
			case 3:   returnString = "Overcast";			break;
			case 4:   returnString = "Light Rain";			break;
			case 5:   returnString = "Heavy Rain";			break;
			case 6:   returnString = "Downpour";			break;
			case 7:   returnString = "Mist";				break;
			case 8:   returnString = "Mist and Rain";		break;
			case 9:   returnString = "Light Fog";			break;
			case 10:  returnString = "Light Fog and Rain";	break;
			case 11:  returnString = "Heavy Fog";			break;
			case 12:  returnString = "Dense Fog";			break;
			case 13:  returnString = "Flurries";			break;
			case 14:  returnString = "Light Snow";			break;
			case 15:  returnString = "Heavy Snow";			break;
			case 16:  returnString = "Blizzard";			break;
			default:  returnString = "Invalid Weather";		break;
		
		}
	
		return returnString;
		
	}
	
	public String analyse_and_return_Temperature(byte temperatureType){
		
		String returnString ="";
		
		switch (temperatureType){
		
			case 0:	 returnString = "Extreme Heat";			break;
			case 1:  returnString = "Hot";					break;
			case 2:  returnString = "Warm";					break;
			case 3:  returnString = "Cool";					break;
			case 4:  returnString = "Cold";					break;
			case 5:  returnString = "Freezing";				break;
			case 6:  returnString = "Extreme Cold";			break;
			default: returnString = "Invalid Temperature"; 	break;
		
		}
	
		return returnString;
		
	}
	
	public String analyse_and_return_GroundCondition(byte groundConditionType){
		
		String returnString ="";
		
		switch (groundConditionType){
		
			case 0:	 returnString = "Very Dry";						break;
			case 1:  returnString = "Dry";							break;
			case 2:  returnString = "Damp";							break;
			case 3:  returnString = "Wet";							break;
			case 4:  returnString = "Muddy";						break;
			case 5:  returnString = "Patchy Snow";					break;
			case 6:  returnString = "Light Snow";					break;
			case 7:  returnString = "Snow";							break;
			case 8:  returnString = "Deep Snow";					break;
			default: returnString = "Invalid Ground Condition"; 	break;
		
		}
	
		return returnString;
		
	}
	
	public String analyse_and_return_EarlyIntel(byte earlyIntelType){
		
		String returnString ="";
		
		switch (earlyIntelType){
		
			case 0:	 returnString = "Allied Force";			break;
			case 1:  returnString = "Axis Force";			break;
			case 2:  returnString = "Neither";				break;
			default: returnString = "Invalid Early Intel"; 	break;
		
		}
	
		return returnString;
		
	}
	
	public String convert_ByteArray_to_String(byte[] byteArray){
		
		return(new String(byteArray));
		
	}
	
	public String[] analyse_and_return_WinThreshold(byte winThresholdType) {
		
		String[] returnString = new String[2];
		
		switch (winThresholdType){
		
			case 0: 	returnString[0] = "Total Defeat"; 
						returnString[1] = "ToD"; break;
		
			case 1: 	returnString[0] = "Major Defeat"; 
						returnString[1] = "MaD"; break;
		
			case 2: 	returnString[0] = "Tactical Defeat";
						returnString[1] = "TaD"; break;
		
			case 3: 	returnString[0] = "Minor Defeat";
						returnString[1] = "MiD"; break;
		
			case 4: 	returnString[0] = "Draw";
						returnString[1] = "Dr"; break;
		
			case 5: 	returnString[0] = "Minor Victory";
						returnString[1] = "MiV"; break;
		
			case 6: 	returnString[0] = "Tactical Victory";
						returnString[1] = "TaV"; break;
		
			case 7: 	returnString[0] = "Major Victory"; 		
						returnString[1] = "MaV"; break;
		
			case 8: 	returnString[0] = "Total Victory"; 		
						returnString[1] = "ToV"; break;
					
			default: 	returnString[0] = "Invalid Victory Level";
						returnString[1] = "INV"; break;
		
		}
		
		return returnString;
		
	}
	
	public String[] analyse_and_return_VictoryLevel(byte VictoryLevelType){
		
		String[] returnString = new String[2];
		
		switch (VictoryLevelType){
		
			case 0: 	returnString[0] = "Total Defeat"; 
						returnString[1] = "ToD"; break;
						
			case 1: 	returnString[0] = "Major Defeat"; 
						returnString[1] = "MaD"; break;
						
			case 2: 	returnString[0] = "Tactical Defeat";
						returnString[1] = "TaD"; break;
						
			case 3: 	returnString[0] = "Minor Defeat";
						returnString[1] = "MiD"; break;
						
			case 4: 	returnString[0] = "Draw";
						returnString[1] = "Dr"; break;
						
			case 5: 	returnString[0] = "Minor Victory";
						returnString[1] = "MiV"; break;
						
			case 6: 	returnString[0] = "Tactical Victory";
						returnString[1] = "TaV"; break;
						
			case 7: 	returnString[0] = "Major Victory"; 		
						returnString[1] = "MaV"; break;
						
			case 8: 	returnString[0] = "Total Victory"; 		
						returnString[1] = "ToV"; break;
						
			case 9:		returnString[0] = "Don't care / determined from overall score";
						returnString[1] = "OS"; break;
						
			default: 	returnString[0] = "Invalid Victory Level";
						returnString[1] = "INV"; break;
		
		}
		
		return returnString;
		
	}
	
	public String analyse_and_return_Size_Modifier(String duration, String battleSize){
		
		int rowIndex, columnIndex;
		
		if (battleSize == "Tiny"){
			
			rowIndex = 0;
			
		} else if (battleSize == "Small"){
			
			rowIndex = 1;
			
		} else if (battleSize == "Medium"){
			
			rowIndex = 2;
			
		} else if (battleSize == "Large"){
			
			rowIndex = 3;
			
		} else if (battleSize == "Huge"){
			
			rowIndex = 4;
			
		} else {
			
			rowIndex = -1;
			
		}
		
		int durationInMinutes = (Integer.valueOf(duration.substring(0,2)) * 60) + Integer.valueOf(duration.substring(3,5));
		
		if ((durationInMinutes > 0) && (durationInMinutes <= 29)){
			
			columnIndex = 0;
			
		} else if ((durationInMinutes >= 30) && (durationInMinutes <= 44)){
			
			columnIndex = 1;
			
		} else if ((durationInMinutes >= 45) && (durationInMinutes <= 59)){
			
			columnIndex = 2;
			
		} else if ((durationInMinutes >= 60) && (durationInMinutes <= 89)){
			
			columnIndex = 3;
			
		} else if (durationInMinutes >= 90){
			
			columnIndex = 4;
			
		} else {
			
			columnIndex = -1;
			
		}
		
		if ((rowIndex >= 0) && (rowIndex <= 4) && (columnIndex >= 0) && (columnIndex <= 4)){
			
			return SMTable[rowIndex][columnIndex];
			
		} else {
			
			return "Invalid Size Modifier";
			
		}
		
		
	}
	
}
