/* 
    CMx2 ScAnCaDe - Scenario Analyser and Campaign Decompiler
    
    Analysing scenarios and campaigns and creating sortable listings 
    containing meta-information about the analysed scenarios/campaigns.
    Also, extract all scenarios belonging to a campaign from the
    campaign file, enabling stand-alone play of these battles.
    Programmed to support the players and scenario designers of the 
    Combat Mission - CMx2 engine WW2 computer games produced by
    Battlefront Inc. (www.battlefront.com)
    
    Copyright (C) 2014  Michael Wellhfer (michael@wellhoefer.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    Change History:
    
    ---------------------------------------------------------------------
    | VERSION |   DATE     |             DESCRIPTION                    |
    |---------|------------|--------------------------------------------|
    |   1.6   | 27/09/2014 | Initial release version. Moved common      |
    |         |            | methods from classes ScenarioReader and    |
    |         |            | CampaignReader into this new utilities     |
    |         |            | class.                                     |
    |---------|------------|--------------------------------------------|

*/

package Parser;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;

public class ParsingUtilities {
	
	public int getIntfromByteArray(byte[] array, int offset) {
		   
		return
	      ((array[offset]   & 0xff) << 24) |
	      ((array[offset+1] & 0xff) << 16) |
	      ((array[offset+2] & 0xff) << 8) |
	       (array[offset+3] & 0xff);
		
	}
	
    public int getShortfromByteArray(byte[] array, int offset){
		
		return
		((array[offset] & 0xff) << 8) |
	       (array[offset+1] & 0xff);
		
	}
    
    public int getShortfromByte(byte inputByte){
		
		return (inputByte & 0xff);
		
	}
    
    public byte getSingleParameterfromFileChannel (ByteBuffer oneByteBuffer, FileChannel inChannel, int startPosition) throws IOException{
		
		oneByteBuffer.clear();
		
	    inChannel.position(startPosition);
	    
	    inChannel.read(oneByteBuffer);
	    
	    oneByteBuffer.rewind();
	    
	    return oneByteBuffer.get(0);
		 
	}
    
    public byte[] gettwoByteArrayfromFileChannel (ByteBuffer twoByteBuffer, FileChannel inChannel, int startPosition) throws IOException{
		
		byte[] intermediateArray = new byte[2];
		
		byte[] intermediateSize = new byte[2];
		
		twoByteBuffer.clear();
		
	    inChannel.position(startPosition);
	    
	    inChannel.read(twoByteBuffer);
	    
	    twoByteBuffer.rewind();
	    
	    twoByteBuffer.get(intermediateArray);
	    
	    intermediateSize[0] = intermediateArray[1];
	    intermediateSize[1] = intermediateArray[0];
	    
	    return intermediateSize;
		
	}
    
    public byte[] getfourByteArrayfromFileChannel (ByteBuffer fourByteBuffer, FileChannel inChannel, int startPosition) throws IOException{
		
		byte[] intermediateArray = new byte[4];
		
		byte[] intermediateSize = new byte[4];	
				
		fourByteBuffer.clear();
		
	    inChannel.position(startPosition);
	    
	    inChannel.read(fourByteBuffer);
	    
	    fourByteBuffer.rewind();
	    
	    fourByteBuffer.get(intermediateArray);
	    
	    intermediateSize[0] = intermediateArray[3];
	    intermediateSize[1] = intermediateArray[2];
	    intermediateSize[2] = intermediateArray[1];
	    intermediateSize[3] = intermediateArray[0];
	    
	    return intermediateSize;
		
	}
    
    public byte[] getflexibleByteArrayfromFileChannel (FileChannel inChannel, int startPosition, int arraySize) throws IOException{
		
		byte[] returnArray = new byte[arraySize];
			
		ByteBuffer flexibleBuffer = ByteBuffer.allocate(arraySize);
		
		flexibleBuffer.clear();
		
	    inChannel.position(startPosition);
	    
	    inChannel.read(flexibleBuffer);
	    
	    flexibleBuffer.rewind();
	    
	    flexibleBuffer.get(returnArray);
	    
	    return returnArray;

	}
    
    public void writebyteArraytoFile (byte[] byteArray, String targetFilename, File targetDir, arrayFileType fileType) throws IOException{		
		
		FileOutputStream fileOut = null;
		
		String fileExtension = "";
		
		switch(fileType){
		
			case BMP: fileExtension = ".bmp"; break;
			case BTT: fileExtension = ".btt"; break;
		
		}
		
		try{
			
			fileOut = new FileOutputStream(targetDir.getPath() + "/" + targetFilename + fileExtension);
			
		    for (int i = 0; i < byteArray.length; i += 1) {
		    	
		    	fileOut.write(byteArray[i]);
		    	
		    }
			
		}finally{
			
			if(fileOut != null){
	    		
	    	    fileOut.close();
	    		
	    	}
			
			byteArray = null;
			
		}
		
	}
    
	public void copyFileToDir (String filename, File targetDir) throws IOException{
		
		File inputFile = new File(System.getProperty("user.dir") + "/" + filename);
	    
		if (inputFile.exists()){
	    	
			File outputFile = new File(targetDir.getName() + "/" + filename);
	    	
			if (!outputFile.exists()){
	    		
				InputStream in = null;
				OutputStream out = null;
				
				try{
				
					in = new FileInputStream(inputFile);
					out = new FileOutputStream(outputFile);
	    		
					byte[] buf = new byte[1024];
					int len;
	    		
					while ((len = in.read(buf)) > 0){
	    			
						out.write(buf, 0, len);
	    		  
					}
				}
				finally{
					
					if(in != null){
						
						in.close();
						
					}
					
					if(out != null){
						
						out.close();
						
					}
					
				}

	    	}
	    	
	    }
		
	}

}
